@@ -1,9 +1,9 @@
 /*
- *  $Id: buildlist.c,v 1.88 2019/08/06 21:08:20 tom Exp $
+ *  $Id: buildlist.c,v 1.93 2020/03/27 20:18:58 tom Exp $
  *
  *  buildlist.c -- implements the buildlist dialog
  *
- *  Copyright 2012-2018,2019	Thomas E. Dickey
+ *  Copyright 2012-2019,2020	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -21,7 +21,7 @@
  *	Boston, MA 02110, USA.
  */
 
-#include <dialog.h>
+#include <dlg_internals.h>
 #include <dlg_keys.h>
 
 /*
@@ -553,11 +553,10 @@ dlg_buildlist(const char *title,
     ALL_DATA all;
     MY_DATA *data = all.list;
     int i, j, k, key2, found, x, y, cur_x, cur_y;
-    int key = 0, fkey;
+    int key, fkey;
     bool save_visit = dialog_state.visit_items;
     int button;
     int cur_item;
-    int was_mouse;
     int name_width, text_width, full_width, list_width;
     int result = DLG_EXIT_UNKNOWN;
     int num_states;
@@ -653,8 +652,7 @@ dlg_buildlist(const char *title,
      * After displaying the prompt, we know how much space we really have.
      * Limit the list to avoid overwriting the ok-button.
      */
-    if (all.use_height + MIN_HIGH > height - cur_y)
-	all.use_height = height - MIN_HIGH - cur_y;
+    all.use_height = height - MIN_HIGH - cur_y;
     if (all.use_height <= 0)
 	all.use_height = 1;
 
@@ -745,6 +743,7 @@ dlg_buildlist(const char *title,
 	int at_top = index2row(&all, moi->top_index, which);
 	int at_end = index2row(&all, -1, which);
 	int at_bot = skip_rows(&all, at_top, all.use_height, which);
+	int was_mouse;
 
 	DLG_TRACE(("# ** state %d:%d top %d (%d:%d:%d) %s\n",
 		   cur_item, item_no - 1,
@@ -1102,9 +1101,7 @@ dlg_buildlist(const char *title,
 		height = old_height;
 		width = old_width;
 		free(prompt);
-		dlg_clear();
-		dlg_del_window(dialog);
-		dlg_mouse_free_regions();
+		_dlg_resize_cleanup(dialog);
 		/* repaint */
 		first = TRUE;
 		goto retry;
@@ -1237,7 +1234,6 @@ dialog_buildlist(const char *title,
 	if ((show_status = dialog_vars.help_status)) {
 	    if (separate_output) {
 		dlg_add_string(help_result);
-		dlg_add_separator();
 	    } else {
 		dlg_add_quoted(help_result);
 	    }
@@ -1259,9 +1255,7 @@ dialog_buildlist(const char *title,
 		}
 	    }
 	}
-	if (dlg_need_separator())
-	    dlg_add_separator();
-	dlg_add_last_key(-1);
+	AddLastKey();
     }
 
     dlg_free_columns(&listitems[0].text, (int) sizeof(DIALOG_LISTITEM), item_no);
