@@ -1,9 +1,9 @@
 /*
- *  $Id: editbox.c,v 1.73 2019/07/24 23:32:46 tom Exp $
+ *  $Id: editbox.c,v 1.79 2020/03/27 21:04:47 tom Exp $
  *
  *  editbox.c -- implements the edit box
  *
- *  Copyright 2007-2018,2019 Thomas E. Dickey
+ *  Copyright 2007-2019,2020 Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -275,7 +275,8 @@ col_to_chr_offset(const char *text, int col)
     return result;
 }
 
-#define SCROLL_TO(target) show_all = scroll_to(pagesize, listsize, &base_row, &thisrow, target)
+#define Scroll_To(target) scroll_to(pagesize, listsize, &base_row, &thisrow, target)
+#define SCROLL_TO(target) show_all = Scroll_To(target)
 
 #define PREV_ROW (*list)[thisrow - 1]
 #define THIS_ROW (*list)[thisrow]
@@ -358,8 +359,8 @@ dlg_editbox(const char *title,
     int result = DLG_EXIT_UNKNOWN;
     int state;
     size_t max_len = (size_t) dlg_max_input(widest_line(*list));
-    char *input, *buffer;
-    bool show_all, show_one, was_mouse;
+    char *buffer;
+    bool show_all, show_one;
     bool first_trace = TRUE;
     WINDOW *dialog;
     WINDOW *editing;
@@ -439,7 +440,8 @@ dlg_editbox(const char *title,
     pagesize = getmaxy(editing);
 
     while (result == DLG_EXIT_UNKNOWN) {
-	int edit = 0;
+	bool was_mouse;
+	char *input;
 
 	if (show_all) {
 	    display_all(editing, *list, thisrow, base_row, listsize, chr_offset);
@@ -513,8 +515,10 @@ dlg_editbox(const char *title,
 	    break;
 	}
 	if (state != sTEXT) {
-	    if (dlg_result_key(key, fkey, &result))
-		break;
+	    if (dlg_result_key(key, fkey, &result)) {
+		if (!dlg_button_key(result, &code, &key, &fkey))
+		    break;
+	    }
 	}
 
 	was_mouse = (fkey && is_DLGK_MOUSE(key));
@@ -555,6 +559,8 @@ dlg_editbox(const char *title,
 	}
 
 	if (state == sTEXT) {	/* editing box selected */
+	    int edit = 0;
+
 	    /*
 	     * Intercept scrolling keys that dlg_edit_string() does not
 	     * understand.
@@ -611,7 +617,7 @@ dlg_editbox(const char *title,
 			    }
 			    --listsize;
 			    --thisrow;
-			    SCROLL_TO(thisrow);
+			    (void) Scroll_To(thisrow);
 
 			    show_all = TRUE;
 			}
@@ -690,7 +696,7 @@ dlg_editbox(const char *title,
 		    chr_offset = 0;
 		    col_offset = 0;
 		    THIS_ROW = tmp;
-		    SCROLL_TO(thisrow);
+		    (void) Scroll_To(thisrow);
 		    show_all = TRUE;
 		} else {
 		    result = dlg_ok_buttoncode(state);
@@ -702,12 +708,10 @@ dlg_editbox(const char *title,
 		/* reset data */
 		height = old_height;
 		width = old_width;
-		dlg_clear();
-		dlg_unregister_window(editing);
-		dlg_del_window(editing);
-		dlg_del_window(dialog);
-		dlg_mouse_free_regions();
 		/* repaint */
+		dlg_del_window(editing);
+		dlg_unregister_window(editing);
+		_dlg_resize_cleanup(dialog);
 		goto retry;
 #endif
 	    case DLGK_TOGGLE:
