@@ -1,9 +1,9 @@
 /*
- *  $Id: formbox.c,v 1.96 2019/07/25 00:06:38 tom Exp $
+ *  $Id: formbox.c,v 1.101 2020/03/27 20:42:19 tom Exp $
  *
  *  formbox.c -- implements the form (i.e., some pairs label/editbox)
  *
- *  Copyright 2003-2018,2019	Thomas E. Dickey
+ *  Copyright 2003-2019,2020	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -502,7 +502,6 @@ dlg_form(const char *title,
 		 : sTEXT);
     int x, y, cur_x, cur_y, box_x, box_y;
     int code;
-    int key = 0;
     int fkey;
     int choice = dlg_default_formitem(items);
     int new_choice, new_scroll;
@@ -610,6 +609,7 @@ dlg_form(const char *title,
 
     while (result == DLG_EXIT_UNKNOWN) {
 	int edit = FALSE;
+	int key;
 
 	if (scroll_changed) {
 	    print_form(form, items, item_no, scrollamt, choice);
@@ -660,8 +660,9 @@ dlg_form(const char *title,
 	}
 
 	key = dlg_mouse_wgetch((state == sTEXT) ? form : dialog, &fkey);
-	if (dlg_result_key(key, fkey, &result))
+	if (dlg_result_key(key, fkey, &result)) {
 	    break;
+	}
 
 	/* handle non-functionkeys */
 	if (!fkey) {
@@ -782,10 +783,8 @@ dlg_form(const char *title,
 		height = old_height;
 		width = old_width;
 		free(prompt);
-		dlg_clear();
+		_dlg_resize_cleanup(dialog);
 		dlg_unregister_window(form);
-		dlg_del_window(dialog);
-		dlg_mouse_free_regions();
 		/* repaint */
 		goto retry;
 #endif
