@@ -1,9 +1,9 @@
 /*
- *  $Id: inputbox.c,v 1.86 2019/08/06 00:50:58 tom Exp $
+ *  $Id: inputbox.c,v 1.91 2020/03/27 20:25:40 tom Exp $
  *
  *  inputbox.c -- implements the input box
  *
- *  Copyright 2000-2018,2019 Thomas E. Dickey
+ *  Copyright 2000-2019,2020 Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -24,7 +24,7 @@
  *	Savio Lam (lam836@cs.cuhk.hk)
  */
 
-#include <dialog.h>
+#include <dlg_internals.h>
 #include <dlg_keys.h>
 
 #define sTEXT -1
@@ -156,9 +156,8 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
 	wsyncup(editor);
 	wcursyncup(editor);
     }
-    while (result == DLG_EXIT_UNKNOWN) {
-	int edit = 0;
 
+    while (result == DLG_EXIT_UNKNOWN) {
 	/*
 	 * The last field drawn determines where the cursor is shown:
 	 */
@@ -196,7 +195,7 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
 	}
 
 	if (state == sTEXT) {	/* Input box selected */
-	    edit = dlg_edit_string(input, &chr_offset, key, fkey, first);
+	    int edit = dlg_edit_string(input, &chr_offset, key, fkey, first);
 
 	    if (edit) {
 		dlg_show_string(editor, input, chr_offset, inputbox_attr,
@@ -245,10 +244,7 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
 		height = old_height;
 		width = old_width;
 		/* repaint */
-		dlg_clear();
-		dlg_del_window(dialog);
-		refresh();
-		dlg_mouse_free_regions();
+		_dlg_resize_cleanup(dialog);
 		goto retry;
 #endif
 	    default:
@@ -259,8 +255,7 @@ dialog_inputbox(const char *title, const char *cprompt, int height, int width,
 	    beep();
 	}
     }
-    dlg_add_separator();
-    dlg_add_last_key(-1);
+    AddLastKey();
 
     dlg_unregister_window(editor);
     dlg_del_window(dialog);
