@@ -1,9 +1,9 @@
 /*
- * $Id: timebox.c,v 1.62 2019/08/05 09:14:59 tom Exp $
+ * $Id: timebox.c,v 1.67 2020/03/27 20:26:21 tom Exp $
  *
  *  timebox.c -- implements the timebox dialog
  *
- *  Copyright 2001-2018,2019   Thomas E. Dickey
+ *  Copyright 2001-2019,2020   Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -21,7 +21,7 @@
  *	Boston, MA 02110, USA.
  */
 
-#include <dialog.h>
+#include <dlg_internals.h>
 #include <dlg_keys.h>
 
 #include <time.h>
@@ -176,7 +176,7 @@ dialog_timebox(const char *title,
     int old_width = width;
 #endif
     BOX hr_box, mn_box, sc_box;
-    int key = 0, key2, fkey;
+    int key, fkey;
     int button;
     int result = DLG_EXIT_UNKNOWN;
     WINDOW *dialog;
@@ -210,11 +210,8 @@ dialog_timebox(const char *title,
 #endif
 
     prompt = dlg_strclone(subtitle);
-    dlg_auto_size(title, prompt, &height, &width, 0, 0);
+    dlg_auto_size(title, prompt, &height, &width, MIN_HIGH, MIN_WIDE);
 
-    height += MIN_HIGH;
-    if (width < MIN_WIDE)
-	width = MIN_WIDE;
     dlg_button_layout(buttons, &width);
     dlg_print_size(height, width);
     dlg_ctl_size(height, width);
@@ -289,6 +286,7 @@ dialog_timebox(const char *title,
 	BOX *obj = (state == sHR ? &hr_box
 		    : (state == sMN ? &mn_box :
 		       (state == sSC ? &sc_box : 0)));
+	int key2;
 
 	button = (state < 0) ? 0 : state;
 	dlg_draw_buttons(dialog, height - 2, 0, buttons, button, FALSE, width);
@@ -363,9 +361,7 @@ dialog_timebox(const char *title,
 		    second = sc_box.value;
 		    /* repaint */
 		    free(prompt);
-		    dlg_clear();
-		    dlg_del_window(dialog);
-		    dlg_mouse_free_regions();
+		    _dlg_resize_cleanup(dialog);
 		    goto retry;
 #endif
 		default:
@@ -426,8 +422,7 @@ dialog_timebox(const char *title,
 	DefaultFormat(buffer, hr_box);
 
     dlg_add_result(buffer);
-    dlg_add_separator();
-    dlg_add_last_key(-1);
+    AddLastKey();
 
     return CleanupResult(result, dialog, prompt, &save_vars);
 }
