@@ -1,9 +1,9 @@
 /*
- *  $Id: treeview.c,v 1.35 2019/08/06 21:35:58 tom Exp $
+ *  $Id: treeview.c,v 1.42 2020/03/27 20:23:03 tom Exp $
  *
  *  treeview.c -- implements the treeview dialog
  *
- *  Copyright 2012-2018,2019	Thomas E. Dickey
+ *  Copyright 2012-2019,2020	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -21,7 +21,7 @@
  *	Boston, MA 02110, USA.
  */
 
-#include <dialog.h>
+#include <dlg_internals.h>
 #include <dlg_keys.h>
 
 #define INDENT 3
@@ -196,12 +196,11 @@ dlg_treeview(const char *title,
 #endif
     ALL_DATA all;
     int i, j, key2, found, x, y, cur_y, box_x, box_y;
-    int key = 0, fkey;
+    int key, fkey;
     int button = dialog_state.visit_items ? -1 : dlg_default_button();
     int choice = dlg_default_listitem(items);
     int scrollamt = 0;
     int max_choice;
-    int was_mouse;
     int use_height;
     int use_width, name_width, text_width, tree_width;
     int result = DLG_EXIT_UNKNOWN;
@@ -291,8 +290,7 @@ dlg_treeview(const char *title,
      * After displaying the prompt, we know how much space we really have.
      * Limit the list to avoid overwriting the ok-button.
      */
-    if (use_height + MIN_HIGH > height - cur_y)
-	use_height = height - MIN_HIGH - cur_y;
+    use_height = height - MIN_HIGH - cur_y;
     if (use_height <= 0)
 	use_height = 1;
 
@@ -357,7 +355,10 @@ dlg_treeview(const char *title,
     dlg_draw_buttons(dialog, height - 2, 0, buttons, button, FALSE, width);
 
     dlg_trace_win(dialog);
+
     while (result == DLG_EXIT_UNKNOWN) {
+	int was_mouse;
+
 	if (button < 0)		/* --visit-items */
 	    wmove(dialog, box_y + choice + 1, box_x + all.check_x + 2);
 
@@ -539,10 +540,7 @@ dlg_treeview(const char *title,
 		height = old_height;
 		width = old_width;
 		/* repaint */
-		dlg_clear();
-		dlg_del_window(dialog);
-		refresh();
-		dlg_mouse_free_regions();
+		_dlg_resize_cleanup(dialog);
 		goto retry;
 #endif
 	    default:
@@ -663,9 +661,7 @@ dialog_treeview(const char *title,
 		}
 	    }
 	}
-	if (dlg_need_separator())
-	    dlg_add_separator();
-	dlg_add_last_key(-1);
+	AddLastKey();
     }
 
     dlg_free_columns(&listitems[0].text, (int) sizeof(DIALOG_LISTITEM), item_no);
