@@ -1,9 +1,9 @@
 /*
- *  $Id: util.c,v 1.274 2019/07/25 00:00:19 tom Exp $
+ *  $Id: util.c,v 1.289 2020/03/27 23:53:01 tom Exp $
  *
  *  util.c -- miscellaneous utilities for dialog
  *
- *  Copyright 2000-2018,2019	Thomas E. Dickey
+ *  Copyright 2000-2019,2020	Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -26,6 +26,7 @@
 
 #include <dialog.h>
 #include <dlg_keys.h>
+#include <dlg_internals.h>
 
 #include <sys/time.h>
 
@@ -81,15 +82,22 @@ DIALOG_VARS dialog_vars;
 
 #ifdef HAVE_COLOR
 #include <dlg_colors.h>
+#ifdef HAVE_RC_FILE2
+#define COLOR_DATA(upr) , \
+	concat(DLGC_FG_,upr), \
+	concat(DLGC_BG_,upr), \
+	concat(DLGC_HL_,upr), \
+	concat(DLGC_UL_,upr), \
+	concat(DLGC_RV_,upr)
+#else /* HAVE_RC_FILE2 */
 #define COLOR_DATA(upr) , \
 	concat(DLGC_FG_,upr), \
 	concat(DLGC_BG_,upr), \
 	concat(DLGC_HL_,upr)
-#else
+#endif /* HAVE_RC_FILE2 */
+#else /* HAVE_COLOR */
 #define COLOR_DATA(upr)		/*nothing */
-#endif
-
-#define DATA(atr,upr,lwr,cmt) { atr COLOR_DATA(upr) RC_DATA(lwr,cmt) }
+#endif /* HAVE_COLOR */
 
 #define UseShadow(dw) ((dw) != 0 && (dw)->normal != 0 && (dw)->shadow != 0)
 
@@ -97,6 +105,7 @@ DIALOG_VARS dialog_vars;
  * Table of color and attribute values, default is for mono display.
  * The order matches the DIALOG_ATR() values.
  */
+#define DATA(atr,upr,lwr,cmt) { atr COLOR_DATA(upr) RC_DATA(lwr,cmt) }
 /* *INDENT-OFF* */
 DIALOG_COLORS dlg_color_table[] =
 {
@@ -139,6 +148,7 @@ DIALOG_COLORS dlg_color_table[] =
     DATA(A_REVERSE,	SEARCHBOX_BORDER2,	searchbox_border2, "Search box border2"),
     DATA(A_REVERSE,	MENUBOX_BORDER2,	menubox_border2, "Menu box border2")
 };
+#undef DATA
 /* *INDENT-ON* */
 
 /*
@@ -153,6 +163,7 @@ add_subwindow(WINDOW *parent, WINDOW *child)
     if (p != 0) {
 	p->normal = parent;
 	p->shadow = child;
+	p->getc_timeout = WTIMEOUT_OFF;
 	p->next = dialog_state.all_subwindows;
 	dialog_state.all_subwindows = p;
     }
@@ -232,6 +243,17 @@ dlg_clear(void)
     dlg_attr_clear(stdscr, LINES, COLS, screen_attr);
 }
 
+#ifdef KEY_RESIZE
+void
+_dlg_resize_cleanup(WINDOW *w)
+{
+    dlg_clear();
+    dlg_put_backtitle();
+    dlg_del_window(w);
+    dlg_mouse_free_regions();
+}
+#endif /* KEY_RESIZE */
+
 #define isprivate(s) ((s) != 0 && strstr(s, "\033[?") != 0)
 
 #define TTY_DEVICE "/dev/tty"
@@ -479,14 +501,16 @@ dlg_color_setup(void)
 	     sizeof(dlg_color_table[0]); i++) {
 
 	    /* Initialize color pairs */
-	    chtype color = dlg_color_pair(dlg_color_table[i].fg,
-					  dlg_color_table[i].bg);
+	    chtype atr = dlg_color_pair(dlg_color_table[i].fg,
+					dlg_color_table[i].bg);
 
-	    /* Setup color attributes */
-	    dlg_color_table[i].atr = ((dlg_color_table[i].hilite
-				       ? A_BOLD
-				       : 0)
-				      | color);
+	    atr |= (dlg_color_table[i].hilite ? A_BOLD : 0);
+#ifdef HAVE_RC_FILE2
+	    atr |= (dlg_color_table[i].ul ? A_UNDERLINE : 0);
+	    atr |= (dlg_color_table[i].rv ? A_REVERSE : 0);
+#endif /* HAVE_RC_FILE2 */
+
+	    dlg_color_table[i].atr = atr;
 	}
 #endif
     } else {
@@ -498,7 +522,7 @@ dlg_color_setup(void)
 int
 dlg_color_count(void)
 {
-    return sizeof(dlg_color_table) / sizeof(dlg_color_table[0]);
+    return TableSize(dlg_color_table);
 }
 
 /*
@@ -680,15 +704,17 @@ dlg_print_listitem(WINDOW *win,
 	attrs[0] = tag_attr;
 
 	dlg_attrset(win, selected ? attrs[3] : attrs[2]);
-	(void) waddnstr(win, text, indx[1]);
+	if (*text != '\0') {
+	    (void) waddnstr(win, text, indx[1]);
 
-	if ((int) strlen(text) > indx[1]) {
-	    limit = dlg_limit_columns(text, climit, 1);
-	    if (limit > 1) {
-		dlg_attrset(win, selected ? attrs[1] : attrs[0]);
-		(void) waddnstr(win,
-				text + indx[1],
-				indx[limit] - indx[1]);
+	    if ((int) strlen(text) > indx[1]) {
+		limit = dlg_limit_columns(text, climit, 1);
+		if (limit > 1) {
+		    dlg_attrset(win, selected ? attrs[1] : attrs[0]);
+		    (void) waddnstr(win,
+				    text + indx[1],
+				    indx[limit] - indx[1]);
+		}
 	    }
 	}
     } else {
@@ -727,6 +753,8 @@ dlg_print_text(WINDOW *win, const char *txt, int cols, chtype *attr)
 	y_origin = y_after = 0;
 	x_origin = x_after = 0;
     } else {
+	y_after = 0;
+	x_after = 0;
 	getyx(win, y_origin, x_origin);
     }
     while (cols > 0 && (*txt != '\0')) {
@@ -773,6 +801,8 @@ dlg_print_text(WINDOW *win, const char *txt, int cols, chtype *attr)
 		case 'n':
 		    *attr = A_NORMAL;
 		    break;
+		default:
+		    break;
 		}
 		++txt;
 	    }
@@ -975,7 +1005,7 @@ justify_text(WINDOW *win,
 	     int *high, int *wide)
 {
     chtype attr = A_NORMAL;
-    int x = (2 * MARGIN);
+    int x;
     int y = MARGIN;
     int max_x = 2;
     int lm = (2 * MARGIN);	/* left margin (box-border plus a space) */
@@ -1754,7 +1784,7 @@ dlg_exit(int code)
     bool overridden = FALSE;
 
   retry:
-    for (n = 0; n < sizeof(table) / sizeof(table[0]); n++) {
+    for (n = 0; n < TableSize(table); n++) {
 	if (table[n].code == code) {
 	    if ((name = getenv(table[n].name)) != 0) {
 		value = strtol(name, &temp, 0);
@@ -1782,8 +1812,8 @@ dlg_exit(int code)
 
 #ifdef NO_LEAKS
     _dlg_inputstr_leaks();
-#if defined(NCURSES_VERSION) && defined(HAVE__NC_FREE_AND_EXIT)
-    _nc_free_and_exit(code);
+#if defined(NCURSES_VERSION) && (defined(HAVE_CURSES_EXIT) || defined(HAVE__NC_FREE_AND_EXIT))
+    curses_exit(code);
 #endif
 #endif
 
@@ -1808,6 +1838,54 @@ dlg_exit(int code)
     }
 }
 
+#define DATA(name) { DLG_EXIT_ ## name, #name }
+/* *INDENT-OFF* */
+static struct {
+    int code;
+    const char *name;
+} exit_codenames[] = {
+    DATA(ESC),
+    DATA(UNKNOWN),
+    DATA(ERROR),
+    DATA(OK),
+    DATA(CANCEL),
+    DATA(HELP),
+    DATA(EXTRA),
+    DATA(ITEM_HELP),
+};
+#undef DATA
+/* *INDENT-ON* */
+
+const char *
+dlg_exitcode2s(int code)
+{
+    const char *result = "?";
+    size_t n;
+
+    for (n = 0; n < TableSize(exit_codenames); ++n) {
+	if (exit_codenames[n].code == code) {
+	    result = exit_codenames[n].name;
+	    break;
+	}
+    }
+    return result;
+}
+
+int
+dlg_exitname2n(const char *name)
+{
+    int result = DLG_EXIT_UNKNOWN;
+    size_t n;
+
+    for (n = 0; n < TableSize(exit_codenames); ++n) {
+	if (!dlg_strcmp(exit_codenames[n].name, name)) {
+	    result = exit_codenames[n].code;
+	    break;
+	}
+    }
+    return result;
+}
+
 /* quit program killing all tailbg */
 void
 dlg_exiterr(const char *fmt, ...)
@@ -1823,6 +1901,13 @@ dlg_exiterr(const char *fmt, ...)
     va_end(ap);
     (void) fputc('\n', stderr);
 
+#ifdef HAVE_DLG_TRACE
+    va_start(ap, fmt);
+    dlg_trace_msg("## Error: ");
+    dlg_trace_va_msg(fmt, ap);
+    va_end(ap);
+#endif
+
     dlg_killall_bg(&retval);
 
     (void) fflush(stderr);
@@ -2158,6 +2243,7 @@ dlg_new_modal_window(WINDOW *parent, int height, int width, int y, int x)
     }
     p->next = dialog_state.all_windows;
     p->normal = win;
+    p->getc_timeout = WTIMEOUT_OFF;
     dialog_state.all_windows = p;
 #ifdef HAVE_COLOR
     if (dialog_state.use_shadow) {
@@ -2170,6 +2256,50 @@ dlg_new_modal_window(WINDOW *parent, int height, int width, int y, int x)
     return win;
 }
 
+/*
+ * dlg_getc() uses the return-value to determine how to handle an ERR return
+ * from a non-blocking read:
+ * a) if greater than zero, there was an expired timeout (blocking for a short
+ *    time), or
+ * b) if zero, it was a non-blocking read, or
+ * c) if negative, an error occurred on a blocking read.
+ */
+int
+dlg_set_timeout(WINDOW *win, bool will_getc)
+{
+    DIALOG_WINDOWS *p;
+    int result = 0;
+    int interval;
+
+    if ((p = find_window(win)) != NULL) {
+	interval = (dialog_vars.timeout_secs * 1000);
+
+	if (will_getc) {
+	    interval = WTIMEOUT_VAL;
+	} else {
+	    result = interval;
+	    if (interval <= 0) {
+		interval = WTIMEOUT_OFF;
+	    }
+	}
+	wtimeout(win, interval);
+	p->getc_timeout = interval;
+    }
+    return result;
+}
+
+void
+dlg_reset_timeout(WINDOW *win)
+{
+    DIALOG_WINDOWS *p;
+
+    if ((p = find_window(win)) != NULL) {
+	wtimeout(win, p->getc_timeout);
+    } else {
+	wtimeout(win, WTIMEOUT_OFF);
+    }
+}
+
 /*
  * Move/Resize a window, optionally with a shadow.
  */
@@ -2230,10 +2360,10 @@ dlg_will_resize(WINDOW *win)
 	    }
 	}
     }
-    wtimeout(win, 0);
-    dlg_trace_msg("# caught %d KEY_RESIZE key%s\n",
-		  1 + caught,
-		  caught == 1 ? "" : "s");
+    dlg_reset_timeout(win);
+    DLG_TRACE(("# caught %d KEY_RESIZE key%s\n",
+	       1 + caught,
+	       caught == 1 ? "" : "s"));
 }
 #endif /* KEY_RESIZE */
 
@@ -2348,7 +2478,7 @@ dlg_strcmp(const char *a, const char *b)
 static bool
 trim_blank(char *base, char *dst)
 {
-    int count = isblank(UCH(*dst));
+    int count = !!isblank(UCH(*dst));
 
     while (dst-- != base) {
 	if (*dst == '\n') {
